#!/usr/bin/env python
""" Services """

from .service import Service  # isort:skip
from .malwarebazaar import MalwareBazaar  # isort:skip
from .shodan import Shodan  # isort:skip
from .virustotal import VirusTotal  # isort:skip

hash_services = [VirusTotal, MalwareBazaar]
ip_services = [Shodan]



#!/usr/bin/env python
"""Module provides human-friendly output from the MalwareBazaar Service"""

import logging

from hurry.filesize import size
from tabulate import tabulate

from ioccheck.cli.formatters.formatter import Formatter
from ioccheck.services import MalwareBazaar

logger = logging.getLogger(__name__)

f_handler = logging.FileHandler("ioccheck.log")
f_handler.setLevel(logging.INFO)

f_format = logging.Formatter("%(asctime)s - %(name)s - %(levelname)s - %(message)s")
f_handler.setFormatter(f_format)

logger.addHandler(f_handler)


class MalwareBazaarFormatter(Formatter):
    """Provide pre-formatted output from the MalwareBazaar Service"""

    def __init__(self, service: MalwareBazaar):
        Formatter.__init__(self, service)

    @property
    def tags(self):
        """Pre-formatted output for user-submitted tags"""
        return ", ".join(self.service.tags) if self.service.tags else None

    @property
    def file_size(self):
        """Pre-formatted output for sample file size"""
        return size(self.service.file_size)

    @property
    def file_type(self):
        """Pre-formatted output for sample file type"""
        return f".{self.service.file_type}"

    @property
    def hashes(self):
        """Pre-formatted output for other hashing algorithm results"""
        results = [[k, v] for k, v in self.service.hashes.items() if v]

        table = [["Type", "Hash"]]

        for result in results:
            table.append(result)

        return tabulate(table, headers="firstrow", tablefmt="fancy_grid")



#!/usr/bin/env python
"""Command-line interface for the ioccheck library"""

import logging
import random
import sys

import click
import pkg_resources
from pyfiglet import Figlet
from termcolor import colored, cprint

from ioccheck.cli.formatters import (MalwareBazaarFormatter, ShodanFormatter,
                                     VirusTotalFormatter)
from ioccheck.exceptions import (InvalidHashException, InvalidIPException,
                                 NoConfiguredServicesException)
from ioccheck.iocs import IP, Hash

asyncio_logger = logging.getLogger("asyncio")
asyncio_logger.setLevel(logging.CRITICAL)


fonts = [
    "slant",
    "banner",
    "basic",
    "bell",
    "block",
    "calgphy2",
    "colossal",
    "cosmic",
    "doom",
    "larry3d",
    "poison",
    "smkeyboard",
    "standard",
    "straight",
    "trek",
]


def shodan_results(ip_addr, heading_color):
    """ Use the ShodanFormatter to print pre-formatted output """
    shodan = ip_addr.reports.shodan

    formatter = ShodanFormatter(shodan)

    cprint("[*] Shodan location data:", heading_color)
    print(formatter.location)

    tags_header = colored("[*] Shodan tags:", heading_color)
    print(f"{tags_header} {formatter.tags}")


def ip_results(ip_addr, heading_color):
    """Print results for an IP address"""
    try:
        shodan_results(ip_addr, heading_color)
    except AttributeError:
        cprint("[!] There was an error displaying the Shodan report.", "red")


def virustotal_results(_hash, heading_color):
    """ Use the VirusTotalFormatter to print pre-formatted output """

    virustotal = _hash.reports.virustotal

    formatter = VirusTotalFormatter(virustotal)

    if formatter.tags:
        tags_heading = colored("[*] VirusTotal tags:", heading_color)
        print(f"{tags_heading} {formatter.tags}")

    investigation_url_heading = colored("[*] VirusTotal URL:", heading_color)
    print(f"{investigation_url_heading} {virustotal.investigation_url}")

    # Make a pretty table of the results
    detections_heading = colored("[*] VirusTotal detections:", "blue")

    print(f"{detections_heading} {formatter.detection_count}")
    print(formatter.detections)

    reputation_heading = colored("[*] VirusTotal reputation:", heading_color)
    print(f"{reputation_heading} {formatter.reputation}")

    if formatter.popular_threat_names:
        threat_names_heading = colored("[*] VirusTotal threat labels:", heading_color)
        print(f"{threat_names_heading} {formatter.popular_threat_names}")


def malwarebazaar_results(_hash, heading_color):
    """ Use the MalwareBazaarFormatter to print pre-formatted output """

    malwarebazaar = _hash.reports.malwarebazaar

    formatter = MalwareBazaarFormatter(malwarebazaar)

    if formatter.tags:
        tags_heading = colored("[*] MalwareBazaar tags:", heading_color)
        print(f"{tags_heading} {formatter.tags}")

    file_size_heading = colored("[*] File details:", heading_color)
    print(
        f"{file_size_heading} {formatter.file_type} | {malwarebazaar.mime_type} ({formatter.file_size})"
    )

    cprint("[*] File hashes:\n", heading_color)
    print(formatter.hashes)


def hash_results(_hash, heading_color):
    """Print results a file hash"""
    hash_algorithm_heading = colored("[*] Hashing algorithm:", heading_color)
    print(f"{hash_algorithm_heading} {_hash.hash_type}")

    try:
        virustotal_results(_hash, heading_color)
    except AttributeError:
        cprint("[!] There was an error displaying the VirusTotal report.", "red")

    try:
        malwarebazaar_results(_hash, heading_color)
    except AttributeError:
        cprint("[!] There was an error diplaying the MalwareBazaar report.", "red")


@click.command()
@click.argument("ioc")
@click.option("--config", required=False, type=str)
def run(ioc, config):
    """Entrypoint for the ioccheck CLI"""

    ioc_types = [
        {
            "name": "file hash",
            "ioc": Hash,
            "exception": InvalidHashException,
            "results": hash_results,
        },
        {
            "name": "public IPv4 or IPv6 address",
            "ioc": IP,
            "exception": InvalidIPException,
            "results": ip_results,
        },
    ]

    figlet = Figlet(font=random.choice(fonts))  # nosec
    heading_color = "blue"

    version = pkg_resources.get_distribution("ioccheck").version

    cprint(
        figlet.renderText("ioccheck"),
    )
    cprint(f"v{version} (https://github.com/ranguli/ioccheck)\n", heading_color)

    printed_ioc = colored(ioc, heading_color)
    print(f"Checking IOC {printed_ioc}.\n")

    check_message = "[*] Checking if IOC is a valid"
    fail_message = "[!] IOC is not a valid"

    for ioc_type in ioc_types:
        try:
            cprint(f"{check_message} {ioc_type.get('name')}.", heading_color)
            ioc = ioc_type.get("ioc")(ioc, config_path=config)
            ioc.check()
            ioc_type.get("results")(ioc, heading_color)
            break
        except ioc_type.get("exception"):
            cprint(f"{fail_message} {ioc_type.get('name')}.", "yellow")
        except NoConfiguredServicesException:
            sys.exit(
                colored(
                    "[!] No configured services available to search that IOC.", "red"
                )
            )
        except FileNotFoundError as error:
            sys.exit(colored(f"[!] {error}", "red"))



#!/usr/bin/env python
"""Module provides a base Service for implenting other services upon"""

import logging
import os
from abc import ABC, abstractmethod
from pathlib import Path
from typing import Any


class Service(ABC):  # pylint: disable=too-few-public-methods
    """Service base class for adding new threat intelligence services"""

    name: str
    ioc: Any
    reputation: Any

    def __init__(self, ioc, api_key: str):
        self.ioc = ioc
        self.api_key = api_key
        self._default_config_path = os.path.join(Path.home(), ".ioccheck")

        self.logger = logging.getLogger(__name__)
        self.logger.setLevel(logging.INFO)

        f_handler = logging.FileHandler("ioccheck.log")
        f_handler.setLevel(logging.INFO)

        f_format = logging.Formatter(
            "%(asctime)s - %(name)s - %(levelname)s - %(message)s"
        )
        f_handler.setFormatter(f_format)

        self.logger.addHandler(f_handler)

    @abstractmethod
    def _get_api_response(self, ioc, api_key: str) -> dict:
        pass

    def __str__(self):
        return self.name



#!/usr/bin/env python
"""Provides support for the Shodan.io service"""

from ipaddress import IPv4Address, IPv6Address
from typing import Optional, Union

import shodan
from backoff import expo, on_exception
from ratelimit import RateLimitException, limits

from ioccheck.services.service import Service


class Shodan(Service):
    """Represents a response from the Shodan.io API"""

    name = "shodan"
    url = "https://shodan.io/host/"
    ioc: Union[IPv4Address, IPv6Address]

    def __init__(self, ioc: Union[IPv4Address, IPv6Address], api_key: str):
        Service.__init__(self, ioc, api_key)

        self.response = self._get_api_response(ioc, api_key)
        self._response_data = self.response.get("data")[0]

    @on_exception(expo, RateLimitException, max_tries=10)
    @limits(calls=15, period=60)
    def _get_api_response(
        self, ioc: Union[IPv4Address, IPv6Address], api_key: str
    ) -> dict:
        client = shodan.Shodan(api_key)
        result = client.host(str(ioc))
        return result

    @property
    def investigation_url(self) -> Optional[str]:
        """The URL a human can use to follow up for more information"""
        return f"{self.url}/{self.ioc}/"

    @property
    def location(self) -> dict:
        """Geolocation data for the IP address"""
        keys = [
            "region_code",
            "postal_code",
            "country_code",
            "city",
            "area_code",
            "country_name",
            "org",
            "isp",
            "asn",
        ]
        return {k: self.response.get(k) for k in keys}

    @property
    def tags(self) -> list:
        """User-submitted tags for the sample from the MalwareBazaar website"""
        return self._response_data.get("tags")

    @property
    def hostnames(self) -> list:
        """Hostnames found for the given IP address"""
        return self.response.get("hostnames")

    @property
    def vulns(self) -> list:
        """CVEs found by the Shodan scanners for the given IP address"""
        return self.response.get("vulns")



#!/usr/bin/env python
"""Module provides human-friendly output from the VirusTotal Service"""

import logging
from typing import Optional

from tabulate import tabulate
from termcolor import colored

from ioccheck.cli.formatters.formatter import Formatter
from ioccheck.services import VirusTotal

logger = logging.getLogger(__name__)

f_handler = logging.FileHandler("ioccheck.log")
f_handler.setLevel(logging.INFO)

f_format = logging.Formatter("%(asctime)s - %(name)s - %(levelname)s - %(message)s")
f_handler.setFormatter(f_format)

logger.addHandler(f_handler)


class VirusTotalFormatter(Formatter):
    """Provide pre-formatted output from the VirusTotal Service"""

    def __init__(self, service: VirusTotal):
        Formatter.__init__(self, service)

    @property
    def reputation(self) -> Optional[str]:
        """Provide pre-formatted output of the community score"""
        if not isinstance(self.service.reputation, int):
            return None

        if self.service.reputation < 0:
            reputation_string = colored(str(self.service.reputation), "red")
        elif self.service.reputation > 0:
            reputation_string = colored(str(self.service.reputation), "green")
        else:
            reputation_string = colored(str(self.service.reputation), "yellow")

        return reputation_string

    @property
    def detection_count(self):
        """Provide pre-formatted output for the number of detections"""
        detection_percent = self.service.detection_coverage * 100

        detection_count_string = f"{self.service.detection_count} engines ({detection_percent:.2g}%) detected this file.\n"
        if self.service.detection_count == 0:
            detection_count_string = colored(detection_count_string, "green")
        elif self.service.detection_count > 0:
            detection_count_string = colored(detection_count_string, "red")

        return detection_count_string

    @property
    def detections(self):
        """Provide pre-formatted output of the detecting A.V engines"""

        table = [["Antivirus", "Detection"]]

        for detection, result in self.service.detections.items():
            if result.get("category") == "malicious":
                table.append([detection, colored(result.get("result"), "red")])

        return tabulate(table, headers="firstrow", tablefmt="fancy_grid")

    @property
    def sandbox_verdicts(self):
        """Provide pre-formatted output of the results from sandbox analysis"""
        result = None
        try:
            result = self.service.response.sandbox_verdicts
        except AttributeError:
            pass

        return result

    @property
    def tags(self):
        """Provide pre-formatted output of user-submitted tags for the sample"""
        return ", ".join(self.service.tags) if self.service.tags else None

    @property
    def popular_threat_names(self):
        """Provide pre-formatted output of popular humand-friendly names"""
        return (
            ", ".join(self.service.popular_threat_names)
            if self.service.popular_threat_names
            else None
        )



#!/usr/bin/env python
"""Module for the MalwareBazaar service"""

from typing import Optional

import requests
from backoff import expo, on_exception
from ratelimit import RateLimitException, limits

from ioccheck.services.service import Service


class MalwareBazaar(Service):
    """ Represents the data from a MalwareBazaar API response"""

    name = "malwarebazaar"
    url = "https://mb-api.abuse.ch/api/v1/"
    ioc: str

    def __init__(self, ioc: str, api_key: str):
        Service.__init__(self, ioc, api_key)
        self.response = self._get_api_response(self.ioc, api_key)
        self._response_data = self.response.get("data")[0]

        if self.response.get("query_status") == "hash_not_found":
            return

        self.is_malicious = None  # self._get_is_malicious(self.response)

    @on_exception(expo, RateLimitException, max_tries=10)
    @limits(calls=15, period=60)
    def _get_api_response(self, ioc: str, api_key: str) -> dict:
        return requests.post(
            self.url,
            data={"query": "get_info", "hash": ioc},
            headers={"API_KEY": api_key},
        ).json()

    @property
    def investigation_url(self) -> Optional[str]:
        """ The URL a human can use to follow up for more information """
        sha256_hash = self.hashes.get("sha256_hash")

        # MalwareBazaar indexes URLs by their SHA256 hash
        if sha256_hash is not None:
            return f"https://bazaar.abuse.ch/sample/{sha256_hash}/"

        return None

    @property
    def hashes(self) -> dict:
        """Other hashes for this file generated by other algorithms"""
        hash_keys = [
            "sha256_hash",
            "sha1_hash",
            "md5_hash",
            "sha3_384_hash",
            "imphash",
            "tlsh",
            "telfhash",
        ]

        return dict(
            (key, value)
            for key, value in self.response.get("data")[0].items()
            if key in hash_keys
        )

    @property
    def signature(self) -> Optional[str]:
        """The signature of the sample returned by MalwareBazaar"""
        return self._response_data.get("signature")

    @property
    def tags(self) -> Optional[list]:
        """User-submitted tags for the sample from the MalwareBazaar website"""
        return self._response_data.get("tags")

    @property
    def file_size(self) -> Optional[int]:
        """Returns the file size of the sample"""
        return self._response_data.get("file_size")

    @property
    def file_type(self) -> Optional[str]:
        """Returns the file extension of the sample"""
        return self._response_data.get("file_type")

    @property
    def mime_type(self) -> Optional[str]:
        """Returns the MIME type of the sample"""
        return self._response_data.get("file_type_mime")



#!/usr/bin/env python
"""Module representing IP addresses"""

import ipaddress
import logging
from dataclasses import dataclass
from ipaddress import IPv4Address, IPv6Address
from typing import List, Optional, Union

from ioccheck.exceptions import InvalidIPException
from ioccheck.iocs.ioc import IOC, IOCReport
from ioccheck.services import Service, Shodan, ip_services

logger = logging.getLogger(__name__)
logger.setLevel(logging.INFO)

asyncio_logger = logging.getLogger("asyncio")
asyncio_logger.propagate = False
asyncio_logger.setLevel(logging.CRITICAL)

f_handler = logging.FileHandler("ioccheck.log")
f_handler.setLevel(logging.INFO)

f_format = logging.Formatter("%(asctime)s - %(name)s - %(levelname)s - %(message)s")
f_handler.setFormatter(f_format)

logger.addHandler(f_handler)


@dataclass
class IPReport(IOCReport):
    """Report representing threat intelligence results for an IP object"""

    shodan: Shodan


class IP(IOC):
    """Type of IOC representing IP addresses

    Attributes:
        hash_type: Indicates the hashing algorithm used by the hash.
        ioc: Represenation of the IOC as a string
        services: Supported services that can be used to investigate the IOC

    """

    def __init__(self, ioc: str, config_path: Optional[str] = None):

        self.services = ip_services
        self.ioc = self.get_ip(ioc)

        IOC.__init__(self, ioc, config_path)

    def check(self, services: Optional[List[Service]] = None):
        """Get threat intelligence information for an IP

        Args:
            services: The threat intelligence services to be checked
        """
        reports = self._get_reports(services)
        self.reports = IPReport(**reports)

    @staticmethod
    def get_ip(ip_addr: str) -> Union[IPv4Address, IPv6Address]:
        """Convert an IPv4 or IPv6 address string into an IPAddress object

        Args:
            ip_addr: The threat intelligence services to be checked

        Returns:
            IP address as an object

        Raises:
            InvalidIPException: If the IP address string can't be converted.
        """
        if not isinstance(ip_addr, str):
            logger.error("%(ip_addr)s is not of type string.")
            raise InvalidIPException

        try:
            ip = ipaddress.ip_address(ip_addr)  # pylint: disable=C0103
        except ValueError as value_error:
            logger.error("%(ip_addr)s could not be converted to IPAddress object.")
            raise InvalidIPException from value_error

        if not ip.is_global:
            logger.error("%(ip_addr)s is not a public IP.")
            raise InvalidIPException

        return ip



#!/usr/bin/env python
""" The IOC module provides base classes for other indicators of compromise """

import configparser
import logging
import os
from dataclasses import dataclass
from pathlib import Path
from typing import List, Optional, Union

from ioccheck.exceptions import (InvalidCredentialsException,
                                 NoConfiguredServicesException)
from ioccheck.services import Service


@dataclass
class IOCReport:
    """Base dataclass for creating indicators of compromise reports """


class IOC:  # pylint: disable=too-few-public-methods,too-many-instance-attributes
    """Base class for creating indicators of compromise classes

    Attributes:
        ioc: The Indicator of Compromise
        config_path: The filepath of the configuration file
        reports: The Indicator of Compromise
        services: All potential services avilable for the IOC type
    """

    def __init__(self, ioc, config_path: Optional[str] = None):

        self.ioc = ioc
        self._default_config_path = os.path.join(Path.home(), ".ioccheck")

        self.logger = logging.getLogger(__name__)
        self.logger.setLevel(logging.INFO)
        self.services: list

        f_handler = logging.FileHandler("ioccheck.log")
        f_handler.setLevel(logging.INFO)

        f_format = logging.Formatter(
            "%(asctime)s - %(name)s - %(levelname)s - %(message)s"
        )
        f_handler.setFormatter(f_format)

        self.logger.addHandler(f_handler)

        if config_path is None:
            self.config_path = self._default_config_path
        else:
            self.config_path = config_path

        if not Path(self.config_path).is_file():
            message = f"File {self.config_path} does not exist."
            self.logger.error(message)
            raise FileNotFoundError(message)

        self.logger.info(
            f"Default config path is {self._default_config_path}, supplied path is {self.config_path}"
        )

        self.reports: IOCReport

    @property
    def credentials(self) -> dict:
        """Credentials for use with API services"""

        config = configparser.ConfigParser()
        config.read(self.config_path)

        credentials: dict = {}

        for section in config.sections():
            values = ",".join(config[section].keys())
            self.logger.info(
                f"Got values {values} for {section} from {self.config_path}."
            )

            credentials.update({section: config[section]["api_key"]})

        return credentials

    @property
    def configured_services(self) -> list:
        """IOC services in the config file with keys"""
        if self.config_path is None:
            self.config_path = self._default_config_path

        if not Path(self.config_path).is_file():
            message = f"File {self.config_path} does not exist"
            self.logger.error(message)
            raise FileNotFoundError(message)

        config = configparser.ConfigParser()
        config.read(self.config_path)

        result = [
            service for service in self.services if service.name in config.sections()
        ]

        if not result:
            self.logger.error("No configured services were found.")
            raise NoConfiguredServicesException

        return result

    def _get_reports(self, services: Optional[Union[List, List[Service]]] = None):

        reports = {}
        # config_path = self._default_config_path if config_path is None else config_path
        report_services = []

        if services is None:
            report_services = self.configured_services
        elif not isinstance(services, list):
            report_services.append(services)
        else:
            report_services = services

        for service in report_services:
            reports.update(self._get_report(self.ioc, service))

        return reports

    def _get_report(self, ioc: str, service: Service) -> dict:
        service = self._single_check(ioc, service)
        return {service.name: service}

    def _single_check(self, ioc, service) -> Service:

        api_key = self.credentials.get(service.name)  # type: ignore

        if not api_key:
            self.logger.error(f"No API keys for {service}")
            raise InvalidCredentialsException

        return service(ioc, api_key)

    def __str__(self):
        return self.ioc



#!/usr/bin/env python
""" Represents response from the VirusTotal API """

from typing import List, Optional

import vt
from backoff import expo, on_exception
from ratelimit import RateLimitException, limits

from ioccheck.services.service import Service


class VirusTotal(Service):
    """ Represents response from the VirusTotal API """

    name = "virustotal"
    url = "https://virustotal.com"

    def __init__(self, ioc, api_key):
        Service.__init__(self, ioc, api_key)

        try:
            self.response = self._get_api_response(self.ioc, api_key)
        except (vt.error.APIError, AttributeError):
            return

    @on_exception(expo, RateLimitException, max_tries=10, max_time=60)
    @limits(calls=4, period=60)
    def _get_api_response(self, ioc: str, api_key: str) -> Optional[dict]:
        client = vt.Client(api_key)
        result = client.get_object(f"/files/{ioc}")

        try:
            return result.to_dict().get("attributes")
        except AttributeError:
            return None

    @property
    def investigation_url(self) -> Optional[str]:
        """ The URL a human can use to follow up for more information """
        return f"{self.url}/gui/file/{self.ioc}/"

    @property
    def detections(self) -> Optional[dict]:
        """The anti-virus providers that detected the hash"""
        try:
            return self.response.get("last_analysis_results")
        except AttributeError:
            return None

    @property
    def detection_coverage(self) -> Optional[float]:
        """The number of A.V providers detecting the sample divided by total providers."""
        if (
            not isinstance(self.detections, dict)
            or self.detections is None
            or not isinstance(self.detection_count, int)
        ):
            return None

        if len(self.detections.keys()) == 0:
            return 0

        return self.detection_count / len(self.detections.keys())

    @property
    def detection_count(self) -> Optional[int]:
        """The number of anti-virus providers available from VirusTotal"""
        if not isinstance(self.detections, dict) or self.detections is None:
            return None

        return len(
            [k for k, v in self.detections.items() if v.get("category") == "malicious"]
        )

    @property
    def reputation(self) -> Optional[int]:
        """VirusTotal community score for a given entry"""
        try:
            return self.response.get("reputation")
        except AttributeError:
            return None

    @property
    def popular_threat_names(self) -> Optional[List[str]]:
        """Human-friendly names that classify a hash as belong to a particular threat"""
        try:
            names = self.response.get(
                "popular_threat_classification"
            ).get(  # type: ignore
                "popular_threat_name"
            )
        except AttributeError:
            return None

        return [name[0] for name in names] if names else None

    @property
    def relationships(self) -> Optional[dict]:
        """Describes how the hash interacts with IPs, domains, etc"""
        try:
            return self.response.get("relationships")
        except AttributeError:
            return None

    @property
    def tags(self) -> Optional[dict]:
        """User-provided tags to classify samples"""
        try:
            return self.response.get("tags") if self.response.get("tags") else None
        except AttributeError:
            return None



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['ioccheck',
 'ioccheck.cli',
 'ioccheck.cli.formatters',
 'ioccheck.iocs',
 'ioccheck.services']

package_data = \
{'': ['*']}

install_requires = \
['backoff>=1.10.0,<2.0.0',
 'click>=7.1.2,<8.0.0',
 'hurry.filesize>=0.9,<0.10',
 'pyfiglet>=0.8.post1,<0.9',
 'ratelimit>=2.2.1,<3.0.0',
 'requests>=2.25.1,<3.0.0',
 'shodan>=1.25.0,<2.0.0',
 'tabulate>=0.8.9,<0.9.0',
 'termcolor>=1.1.0,<2.0.0',
 'vt-py>=0.6.1,<0.7.0']

entry_points = \
{'console_scripts': ['ioccheck = ioccheck.cli:run']}

setup_kwargs = {
    'name': 'ioccheck',
    'version': '0.3.1',
    'description': 'A tool for simplifying the process of researching IOCs.',
    'long_description': None,
    'author': 'ranguli',
    'author_email': 'hello@joshmurphy.ca',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7.3,<4.0.0',
}


setup(**setup_kwargs)



#!/usr/bin/env python
"""Module provides human-friendly output from the Shodan.io Service"""

import logging

from tabulate import tabulate
from termcolor import colored

from ioccheck.cli.formatters.formatter import Formatter
from ioccheck.services import Shodan

logger = logging.getLogger(__name__)

f_handler = logging.FileHandler("ioccheck.log")
f_handler.setLevel(logging.INFO)

f_format = logging.Formatter("%(asctime)s - %(name)s - %(levelname)s - %(message)s")
f_handler.setFormatter(f_format)

logger.addHandler(f_handler)


class ShodanFormatter(Formatter):
    """Provide pre-formatted output from the Shodan.io Service"""

    def __init__(self, service: Shodan):
        Formatter.__init__(self, service)

    @property
    def tags(self):
        """Pre-formatted output for user-submitted tags"""
        return ", ".join(self.service.tags) if self.service.tags else None

    @property
    def location(self):
        """Pre-formatted output for geolocation data"""

        location = self.service.location
        table = []

        headings = {
            "City": location.get("city"),
            "Country": location.get("country_name"),
            "Organization": location.get("org"),
            "ISP": location.get("isp"),
            "Hostnames": ", ".join(self.service.hostnames),
            "ASN": location.get("asn"),
        }

        for title, value in headings.items():
            if value:
                table.append([colored(title, "blue"), value])

        return tabulate(table, tablefmt="fancy_grid")



#!/usr/bin/env python
"""Module providing custom types for ioccheck"""

from dataclasses import dataclass


class IOCType:  # pylint: disable=too-few-public-methods
    """Base class for representing a type of IOC"""


@dataclass
class HashType(IOCType):
    """Representation of file hash"""

    name: str
    regex: str

    def __str__(self) -> str:
        return self.name


SHA256 = HashType(name="SHA256", regex=r"^[A-Fa-f0-9]{64}$")
SHA1 = HashType(name="SHA256", regex=r"^[A-Fa-f0-9]{40}$")
MD5 = HashType(name="MD5", regex=r"^[a-f0-9]{32}$")

hash_types = [SHA256, SHA1, MD5]



#!/usr/bin/env/python
"""Module for the formatter base class"""

import logging

from ioccheck.services import Service


class Formatter:  # pylint: disable=too-few-public-methods
    """Base class for creating human-friendly threat intelligence API output"""

    def __init__(self, service: Service):
        self.service = service

        self.logger = logging.getLogger(__name__)
        self.logger.setLevel(logging.INFO)

        f_handler = logging.FileHandler("ioccheck.log")
        f_handler.setLevel(logging.INFO)

        f_format = logging.Formatter(
            "%(asctime)s - %(name)s - %(levelname)s - %(message)s"
        )
        f_handler.setFormatter(f_format)

        self.logger.addHandler(f_handler)



"""Provides ability to generate pre-formatted human-friendly output from services."""

from ioccheck.cli.formatters.formatter import Formatter
from ioccheck.cli.formatters.malwarebazaar import MalwareBazaarFormatter
from ioccheck.cli.formatters.shodan import ShodanFormatter
from ioccheck.cli.formatters.virustotal import VirusTotalFormatter



#!/usr/bin/env python
"""Module representing file hashes"""


import re
from dataclasses import dataclass
from typing import List, Optional

from ioccheck.exceptions import InvalidHashException
from ioccheck.ioc_types import MD5, SHA256, HashType, hash_types
from ioccheck.iocs.ioc import IOC, IOCReport
from ioccheck.services import MalwareBazaar, Service, VirusTotal, hash_services


@dataclass
class HashReport(IOCReport):
    """Report representing threat intelligence results for a Hash object

    Attributes:
        virustotal: Results from the VirusTotal API
        malwarebazaar: Results from the MalwareBazaar API
    """

    virustotal: VirusTotal = None  # type: ignore
    malwarebazaar: MalwareBazaar = None  # type: ignore


class Hash(IOC):  # pylint: disable=too-few-public-methods,too-many-instance-attributes
    """Type of IOC representing file hashes

    Attributes:
        hash_type: Indicates the hashing algorithm used by the hash.
        ioc: Represenation of the IOC as a string
        services: Supported services that can be used to investigate the IOC
        is_sha256: Whether or not the hash is a SHA256 hash.
        is_md5: Whether or not the hash is an MD5 hash.

    """

    def __init__(
        self,
        ioc: str,
        hash_type: Optional[HashType] = None,
        config_path: Optional[str] = None,
    ):

        IOC.__init__(self, ioc, config_path)

        self.hash_type = hash_type
        self._supported_types = ", ".join([str(hash_type) for hash_type in hash_types])
        self.invalid_hash_msg = f"Hash is not a supported hash type. Supported types are {self._supported_types}"
        self.services = hash_services

        if not isinstance(self.ioc, str):
            self.logger.error(f"{self.ioc} is not of type str")
            raise InvalidHashException

        if hash_type:
            if not self._check_hash_type(hash_type.regex, ioc):
                self.logger.error(
                    f"{ioc} is not a valid hash because it doesn't match regex {hash_type.regex}"
                )
                raise InvalidHashException(self.invalid_hash_msg)
        else:
            self.hash_type = self._guess_hash_type(self.ioc)

        self.is_sha256 = bool(self.hash_type == SHA256)
        self.is_md5 = bool(self.hash_type == MD5)

    def _guess_hash_type(self, file_hash: str) -> HashType:
        """Guesses the type of a file hash.

        Args:
            file_hash: The file hash to guess the type of

        Returns:
            The type of the file hash

        Raises:
            InvalidHashException: If the input hash matches no known type
        """

        for hash_type in hash_types:
            self.logger.info(f"Trying hash {hash_type}")
            if self._check_hash_type(hash_type.regex, file_hash):
                self.logger.info(f"Hash {file_hash} is of type {hash_type}")
                return hash_type

        self.logger.error(f"The type of hash {file_hash} could not be guessed!")
        raise InvalidHashException(self.invalid_hash_msg)

    @staticmethod
    def _check_hash_type(hash_regex: str, file_hash: str) -> bool:
        """Validate that a given hash matches a regex.

        Args:
            hash_regex: The regex to identify a hashes type
            file_hash: The hash to be checked

        Returns:
            Whether or not the hash matches the regex.
        """

        return bool(re.match(hash_regex, file_hash))

    def check(self, services: Optional[List[Service]] = None):
        """Get threat intelligence information for a hash

        Args:
            services: The threat intelligence services to be checked
        """

        reports = self._get_reports(services)
        self.reports = HashReport(**reports)



#!/usr/bin/env python
""" Exceptions for ioccheck """


class InvalidHashException(Exception):
    """Raised if a given hash is invalid.

    If an hash string doesn't match any of the regexes provided by
    the support hash types, this exception is thrown.
    """


class InvalidIPException(Exception):
    """Raised if a given IP is invalid.

    If a string isn't a valid IP address, or is not a public
    IP address, this exception is thrown.
    """


class NoConfiguredServicesException(Exception):
    """Raised if there are no services setup with credentials.

    If a config file has no entries for services, this
    exception is thrown
    """


class InvalidCredentialsException(Exception):
    """Raised if there is some kind of error with credentials.

    If an API returns an authentication error, or no valid credential
    is retrieved for a service from the configuration file, this
    exception is thrown.
    """



#!/usr/bin/env python

""" Various indicators of compromise """

from .ioc import IOC, IOCReport  # isort:skip
from .hash import Hash, HashReport  # isort:skip
from .ip import IP, IPReport  # isort:skip
